/* Convenience funcs for calculating angles. Angles increase going clockwise and are zero at the x = 1, y = 0. */

var angles = {

	degreesFor: function(x1, y1, x2, y2) {
		/* Gets angle of line going from (x1, y1) to (x2, y2)*/
		var dx = x2 - x1;
		var dy = y2 - y1;
		var a = Math.atan2(dy, dx) * 180 / Math.PI;
		return angles.normalize(a);
	},

	getSmallestDif: function(deg1, deg2) {
		/* Return the smallest degree difference going from deg1 to deg2.
			Answer will be negative if going counter clockwise and positive otherwise.
		Examples
			getSmallestDif(0, 60) -> 60
			getSmallestDif(0, 270) -> -90
			getSmallestDif(30, 0) -> -30
			getSmallestDif(0, 180) -> 180
		*/
		var dif = deg2 - deg1;
		var norm = this.normalize(dif);
		if(norm > 180) {
			norm -= 360;
		}
		return norm;
	},

	normalize: function(angle) {
		/* Convert to corresponding angle in 0 - 360 degree range */
		if(angle < 0) {
			return (angle % 360) + 360;
		} else {
			return angle % 360;
		}
	},

};
