/* 
View to model the circular timer.

Notes
	- Can't have buttons cursor set to pointer b/c then it shows a pointer event when out of bouunds.
*/

var CircleView = function() {
	//Properties
	this.digital = new DigitalView();
	this.digital.main.addClass('circle-time-bar');
	this.wedge = new Wedge();
	this.wedge.setRotationDegrees(-90);
	this.wedge.jq.css({
		pointerEvents: 'none',
		width: '100%',
		height: '100%',
	});
	this.hand = $('<div/>')
		.addClass('hand circle-hand');
	this.outerHand = $('<div/>')
		.addClass('circle-hand-outer')
		.append(this.hand);
	this.staticHand = $('<div/>').addClass('hand circle-static-hand');
	this.pause = this.createBtn('img/pause.png', 'img/pause2.png', this._onPause, 'circle-play-btn');
	this.play = this.createBtn('img/play.png', 'img/play2.png', this._onPlay, 'circle-play-btn');
    this.reset = this.createBtn('img/refresh.png', 'img/refresh2.png', this._onReset, 'circle-reset-btn');
	this.v1 = $('<div/>')
		.addClass('circle-v1')
		.append(this.pause, this.play, this.reset, this.digital.main);
	this.v2 = $('<div/>')
		.addClass('circle-v2')
		.append(this.wedge.jq, this.staticHand);
	this.main = $('<div/>')
		.addClass('circle-view')
		.append(this.v1, this.v2, this.outerHand);
	this.onButton = null; // function(CircleView v, JQ button, Event event);
	this._prog = 1;
	this._progShown = true;
	this._borderColor = 'white';
	this.setProgColor('#1ac727');
	this.setInnerColor('white');
	this.setPlayShown(true);
};

CircleView.prototype = {

	//Create
	createBtn: function(src1, src2, click, className) {
		var img1 = $('<img/>')
			.attr('src', src1)
			.attr('draggable', false)
			.css({
				position: 'absolute',
			});
		var img2 = $('<img/>')
			.attr('src', src2)
			.attr('draggable', false)
			.css({
				position: 'absolute',
			})
			.hide();
		var btn = $('<div/>')
			.addClass('unselectable')
            .addClass(className)
			.append(img1, img2)
			.click(click.bind(this));
		btn.img1 = img1;
		btn.img2 = img2;
		return btn;
	},
    
	//Layout
	layout: function() {
		
		//DigitalView
		this.layoutDigital();

		//v1 & v2
		var border = 0.02 * this.main.width();
		this.v1.css({
			border: border + 'px solid ' + this._borderColor,
		});
		var x1 = 3 * border / 4;
		this.v2.css({
			left: x1,
			top: x1,
			right: x1,
			bottom: x1,
		});

		//Wedges
		this.wedge.setOuterBorderWidth(this.hand.width());
		
		//Hand
		var r = this.v2.width() / 2;
		var r1 = this.outerHand.width() / 2 - this.hand.width() / 2;
		this.outerHand.css({
			top: x1 - r1,
			left: this.main.width() / 2 - this.outerHand.width() / 2,
			transformOrigin: 'center ' + (r + r1) + 'px', 
		});
		this.staticHand.css({
			left: this.v2.width() / 2 - this.hand.width() / 2,
		});

		//Btns
		var w2 = 0.2 * this.play.width();
		var right = 0.16 * this.play.width();
		var top = 0.08 * this.play.width();
		var ims = [this.play.img1, this.play.img2, this.pause.img1, this.pause.img2];
		for(var i = 0; i < ims.length; i++) {
			ims[i].width(w2).height(w2).css({
				right: right,
				top: top,
			});
		}
        
        var left = 0.16 * this.reset.width();
		ims = [this.reset.img1, this.reset.img2];
		for(var i = 0; i < ims.length; i++) {
			ims[i].width(w2).height(w2).css({
				left: left,
				top: top,
			});
		}

		//Prog
		this.setProg(this._prog);
	},

	layoutDigital: function(digital) {
		var v = this.digital;
		v.updateBorder();
		v.setFontSize(0.14 * this.main.height());
		v.box.css({
			top: '27%',
			bottom: '27%',
		});
		v.layoutInputs(72, 44, 16, 12);
		v.layoutArrows(77, 56, 29, -4, 75, 15);
		v.colon1.css({
			right: '67%',
		});
		v.colon2.css({
			right: '39%',
		});	
	},

	//Props
	isPlayShown: function() {
		return this.play.is(':visible');
	},

	setPlayShown: function(shown) {
		if(shown) {
			this.play.show();
			this.pause.hide();
		} else {
			this.play.hide();
			this.pause.show();
		}
	},

	setInnerColor: function(color) {
		var b = '1px solid ' + color;
		this.play.css({
			borderTop: b,
		});
		this.pause.css({
			borderTop: b,
		});
		this.reset.css({
			borderTop: b,
			borderLeft: b,
		});
		this.main.css({
			// border: '2px solid ' + color,
		});
		this.hand.css({
			backgroundColor: color,
		});
		this.staticHand.css({
			backgroundColor: color,
		});
		this.digital.setRegTextColor(color);
	},

	setLightBtnsShown: function(shown) {
		this.setLightBtnShown(this.play, shown);
		this.setLightBtnShown(this.pause, shown);
        this.setLightBtnShown(this.reset, shown);
		this.digital.setLightBtnsShown(shown);
	},

	setLightBtnShown: function(btn, shown) {
		if(shown) {
			btn.img1.show();
			btn.img2.hide();
		} else {
			btn.img1.hide();
			btn.img2.show();
		}
	},

	//Progress
	setProgWidth: function(w) {
		this.wedge.setOuterBorderWidth(w);
	},

	setProgColor: function(color) {
		this.wedge.setOuterBorderColor(color);
	},

	setProgShown: function(shown) {
		this._progShown = shown;
		var views = [
			this.wedge.jq,
			this.outerHand,
			this.staticHand,
		];
		for(var i = 0; i < views.length; i++) {
			if(this._progShown) {
				views[i].show();
			} else {
				views[i].hide();
			}
		}
	},

	prog: function() {
		return this._prog;
	},

	setProg: function(prog) {
		/* 0 - 1 range. 0.25 correspond to top left corner being filled. 1 = full. */
		this._prog = prog;

		//Move Wedges
		this.wedge.setSpanDegrees(prog * 360);

		//Move dot
		var deg = -360 * (1 - prog);
		var s = 'rotate(' + deg + 'deg)';
		this.outerHand.css({
			'-webkit-transform': s,
			'transform': s,
		});
	},

	progFor: function(degrees) {
		degrees = angles.normalize(degrees);
		if(degrees >= 270) {
			return (0.25 + degrees  / 360) % 1;
		} else {
			return 0.25 + degrees  / 360;
		}
	},

	degrees: function() {
		return angles.normalize(360 * (this._prog - 0.25));
	},

	setDegrees: function(degrees) {
		this.setProg((0.25 + degrees  / 360) % 1);
	},

	degreesFor: function(x, y) {
		return angles.degreesFor(this.main.width() / 2, this.main.height() / 2, x, y);
	},

	//Events
	eventIsInBounds: function(event) {
		var offset = this.main.offset();
		var w = this.main.width() / 2;
		var x = event.pageX - offset.left - w;
		var y = event.pageY - offset.top - w;
		return x * x + y * y < w * w;
	},

	_onPlay: function(event) {
		if(this.onButton) {
			this.onButton(this, this.play, event);
		}
	},

	_onPause: function(event) {
		if(this.onButton) {
			this.onButton(this, this.pause, event);
		}
	},

	_onReset: function(event) {
		if(this.onButton) {
			this.onButton(this, this.reset, event);
		}
	},
	
};
