/* Object which keeps track of a point in time. Has special functionality to listen to timeouts and increment time as well. */

var Counter = function(onTimeoutUpdate) {
	this.onTimeoutUpdate = onTimeoutUpdate; //function to call when timeout happens
	this._time = 0; //millisecs, main time
	this._lastUpdate = 0; //clock time at last update.
	this._timeoutId = null;
	this._isCountingUp = false;
};

Counter.prototype = {
	
	//Props
	time: function() {
		return this._time;
	},

	setTime: function(time) {
		this._time = time;
	},

	hours: function() {
		return Math.floor(this._time / 3600000);
	},

	mins: function() {
		return Math.floor((this._time  % 3600000) / 60000);
	},

	secs: function() {
		return Math.floor((this._time  % 60000) / 1000);
	},

	millisecs: function() {
		return this._time % 1000;
	},

	ceilTime: function() {
		/* Rounds up to the nearest sec so that we don't get to a case where there is 0.5 secs left but the timer shows 0:00. */
		var h = this.hours();
		var m = this.mins();
		var s = Math.ceil((this._time  % 60000) / 1000);
		if(s >= 60) {
			s -= 60;
			m += 1;
		}
		if(m >= 60) {
			m -= 60;
			h += 1;
		}
		return {
			hours: h,
			mins: m,
			secs: s,
		}
	},

	setTime2: function(hours, mins, secs, millisecs) {
		this._time = (hours * 3600 + mins * 60 + secs) * 1000 + millisecs;
	},

	addHours: function(n) {
		this._time += n * 3600000;
	},

	addMins: function(n) {
		this._time += n * 60000;
	},

	addSecs: function(n) {
		this._time += n * 1000;
	},

	bound: function(low, high) {
		if(this._time < low) {
			this._time = low;
		}
		if(this._time > high) {
			this._time = high;
		}
	},

	//Ops
	isPlaying: function() {
		return this._timeoutId !== null;
	},

	play: function() {
		if(this._timeoutId === null || (!this._isCountingUp && this._time <= 0)) {
			this._lastUpdate = millis();
			this._postNext();
		}
	},

	stop: function() {
		if(this._timeoutId !== null) {
			this._updateTime();
			clearTimeout(this._timeoutId);
			this._timeoutId = null;
		}
	},

	isCountingUp: function() {
		return this._isCountingUp;
	},

	setCountingUp: function(isUp) {
		if(this._isCountingUp === isUp) {
			return;
		}
		if(this.isPlaying()) {
			this.stop();
			this._isCountingUp = isUp;
			this.play();
		} else {
			this._isCountingUp = isUp;
		}
	},

	//Other
	_updateTime: function() {
		var m = millis();
		var dt = m - this._lastUpdate;
		if(this._isCountingUp) {
			this._time += dt;
		} else {
			this._time -= dt;
		}
		this._lastUpdate = m;
	},

	_onTimeout: function() {
		this._updateTime();
		if(this.onTimeoutUpdate) {
			this.onTimeoutUpdate();
		}
		if(this.isPlaying()) {
			this._postNext();
		}
	},

	_postNext: function() {
		var dt = 20;
		this._timeoutId = setTimeout(this._onTimeout.bind(this), dt);
	},

};
