/*
Rectangular timer view.

Properties
	hand: JQ -- draggable circle on bottom of view.
	reset: JQ
	play: JQ -- wraps both the white and black images.
	pause: JQ -- wraps both the white and black images.
	prog: JQ -- green prog bar
	progback: JQ -- background of progress bar. forms border
	digital : DigitalView
	inner: JQ -- wraps everything except progress bar
	_prog : Number -- progress value.
 */

var RectView = function() {
	this.hand = $('<div/>')
		.addClass('hand rect-hand')
	this.outerHand = $('<div/>')
		.addClass('rect-hand-outer')
		.append(this.hand);
    this.reset = this.createBtn('img/refresh.png', 'img/refresh2.png', 'rect-reset');
	this.play = this.createBtn('img/play.png', 'img/play2.png', 'rect-play-pause');
	this.pause = this.createBtn('img/pause.png', 'img/pause2.png', 'rect-play-pause');
	this.prog = $('<div/>')
		.addClass('rect-prog rect-prog-bar')
		.append(this.outerHand);
	this.progBack = $('<div/>')
		.addClass('rect-prog rect-prog-back')
		.append(this.prog);
	this.digital = new DigitalView();
	this.digital.main.addClass('rect-digital-view');
	this.inner = $('<div/>')
		.addClass('rect-inner')
		.append(this.reset, this.pause, this.play, this.digital.main);
	this.main = $('<div/>')
		.addClass('rect-view')
		.append(this.inner, this.progBack);
	this._prog = 0;
	this.setAlternateColor('white');
	this.setPlayShown(true);
};

RectView.prototype = {

	createBtn: function(src1, src2, className) {
		var img1 = $('<img/>')
			.attr('src', src1)
			.attr('draggable', false)
			.addClass('centered');
		var img2 = $('<img/>')
			.attr('src', src2)
			.attr('draggable', false)
			.addClass('centered')
			.hide();
		var btn = $('<div/>')
			.addClass('centered-container')
            .addClass(className)
			.append(img1, img2);
		btn.img1 = img1;
		btn.img2 = img2;
		return btn;
	},

	layout: function() {
		this.layoutDigital();
		this.hand.width(this.hand.height());
		this.outerHand.width(1.5 * this.outerHand.height());
		
		//Btns
		var w2 = 0.4 * this.play.width();
		var ims = [this.play.img1, this.play.img2, this.pause.img1, this.pause.img2];
		for(var i = 0; i < ims.length; i++) {
			ims[i].width(w2).height(w2);
		}
        
        w2 = 0.2 * this.reset.width();
		ims = [this.reset.img1, this.reset.img2];
		for(var i = 0; i < ims.length; i++) {
			ims[i].width(w2).height(w2);
		}

		//Prog
		this.setProg(this._prog);
	},

	layoutDigital: function(digital) {
		var v = this.digital;
		var width = v.main.height() * 1.54;
		v.main.css({
			left: 0.63 * this.main.width() - width / 2,
			width: width,
		});
		v.setFontSize(0.34 * this.main.height());
		v.updateBorder();
		v.layoutInputs(74, 40, 6, 17);
		v.layoutArrows(80, 55, 22, 5, 70, 15);
		v.box.css({
			top: '30%',
			bottom: '30%',
		});
		v.colon1.css({
			right: '67%',
		});
		v.colon2.css({
			right: '34%',
		});
	},

	isPlayShown: function() {
		return this.play.is(':visible');
	},

	setPlayShown: function(shown) {
		if(shown) {
			this.play.show();
			this.pause.hide();
		} else {
			this.play.hide();
			this.pause.show();
		}
	},

	setLightBtnsShown: function(shown) {
		this.setLightBtnShown(this.pause, shown);
		this.setLightBtnShown(this.play, shown);
        this.setLightBtnShown(this.reset, shown);
		this.digital.setLightBtnsShown(shown);
	},

	setLightBtnShown: function(btn, shown) {
		if(shown) {
			btn.img1.show();
			btn.img2.hide();
		} else {
			btn.img1.hide();
			btn.img2.show();
		}
	},

	setBorderColor: function(color) {
		var b = '1px solid ' + color;
		this.inner.css({
			border: b,
		});
		this.progBack.css({
			border: b,
		});
		this.play.css({
			borderRight: b,
			borderBottom: b,
		});
		this.pause.css({
			borderRight: b,
			borderBottom: b,
		});
		this.reset.css({
			borderRight: b,
		});
		
	},

	setAlternateColor: function(color) {
		this.setBorderColor(color);
		this.digital.setRegTextColor(color);
		this.reset.css({
			color: color,
		});
		this.hand.css({
			backgroundColor: color,
		})
	},

	//Prog
	isProgShown: function() {
		return this.prog.is(':visible');
	},

	setProgShown: function(shown) {
		if(shown) {
			this.progBack.show();
		} else {
			this.progBack.hide();
		}
		this.layout();
	},

	prog: function() {
		/* 0 - 1. 0 correspondss to no green left. */
		return this._prog;
	},

	setProg: function(p) {
		this._prog = p;
		this.prog.width(p * this.main.width());
	},

};
