/* Top Level Controller */
var Timer = function() {
	localize();
	
	//Properties
	this.counter = new Counter(this.onCounterUpdate.bind(this)); //Main model object.
	this.view = new TimerView();
	this.handDown = false; //true if mouse is currently down on the second hand circle.
	this.wasPlayingPreDrag = false;
	this.lastSetTime = 0; //we track this time to update the timer when reset is clicked.
	this.aud = new Audio();
	this.selection = new SelectionListener(this.onSelectionChange.bind(this));
	this.longPress = new LongPress(this.onLongPressInc.bind(this));
	this.resizeAction = null;

	//Initialize
	$('body').append(this.view.main);
	if(!this.read()) {
		var flash = this.readFlash();
		if(flash) {
			this.applyFlash(flash);
		} else {
			this.setTime(0, 5, 0);
		}
	}
	this.view.layout();
	this.addListeners();
	this.updateThumb();
	this.updateMinSizes();
};

Timer.prototype = {

	//Init
	addListeners: function() {
		/* View Events */
		$(window).resize(this.onResize.bind(this))
			.focusout(this.onFocusOut.bind(this))
			.focusin(this.onFocusIn.bind(this));
		var v = this.view;
		v.settingsBtn.back.click(this.onSettingsClick.bind(this));
		v.colorBox.box.click(this.onColorBox.bind(this));
		v.main.mousemove(this.onMouseMove.bind(this))
			.mouseup(this.onMouseUp.bind(this))
			.mouseleave(this.onMouseUp.bind(this))
			.click(this.onMainClick.bind(this));
			
		var rv = v.rect;
		rv.play.click(this.onPlayClick.bind(this));
		rv.pause.click(this.onPauseClick.bind(this));
		rv.reset.click(this.onResetClick.bind(this));
		this.addDigitalViewListeners(rv.digital);
		rv.outerHand.mousedown(this.onHandMouseDown.bind(this));
		
		var cv = v.circle;
		cv.onButton = this.onCircleButton.bind(this);
		this.addDigitalViewListeners(cv.digital);
		cv.outerHand.mousedown(this.onHandMouseDown.bind(this));

		for(var i = 0; i < v.drops.length; i++) {
			var drop = v.drops[i];
			drop.onClick = this.onDropClick.bind(this);
			drop.onOptionClick = this.onDropOptionClick.bind(this);
		}

		var cp = v.colorPicker;
		cp.onXClick = this.onColorPickerX.bind(this);
		cp.onWheelUp = this.onColorPickerWheelUp.bind(this);
		cp.onDrag = this.onColorPickerDrag.bind(this);
		cp.onHexClick = this.onColorPickerHex.bind(this);
	},

	addDigitalViewListeners: function(digital) {
		digital.onArrowMouseDown = this.onArrowMouseDown.bind(this);
		digital.onArrowMouseUp = this.onArrowMouseUp.bind(this);
		digital.onArrowMouseLeave = this.onArrowMouseLeave.bind(this);
		digital.onKeyDown = this.onInputKeyDown.bind(this);
		digital.onInputMouseDown = this.onInputMouseDown.bind(this);
		digital.onInputFocusOut = this.onInputFocusOut.bind(this);
	},

	//Props
	setTime: function(hours, mins, secs) {
		this.counter.setTime2(hours, mins, secs, 0);
		this.counter.bound(0, Timer.MAX_TIME);
		this.lastSetTime = this.counter.time();
		this.updateViewTimes();
	},

	setColor: function(rgb) {
		this.view.setColor(rgb);
		this.write();
		this.updateThumb();
	},

	setCountingUp: function(isCountingUp) {
		this.counter.setCountingUp(isCountingUp);
		this.view.countDrop.setSelectedIndex(isCountingUp ? 1 : 0);
		this.view.setProgShown(!isCountingUp);
		this.write();
	},

	updateViewTimes: function() {
		var ceil = this.counter.ceilTime();
		var isPlaying = this.counter.isPlaying();
		this.view.setTime(ceil.hours, ceil.mins, ceil.secs);
		this.view.setIsPlaying(isPlaying);
		this.view.setArrowsShown(!isPlaying);
		this.updateProg();
		this.write();
	},

	updateProg: function() {
		var prog = 0;
		if(this.lastSetTime > 0) {
			//Get ceil values to show bar inching along.
			prog = Math.ceil((this.counter.time() / this.lastSetTime) * 200) / 200;
		}
		this.view.setProg(prog);
	},

	play: function() {
		var up = this.counter.isCountingUp();
		var time = this.counter.time();
        if ((up && time < Timer.MAX_TIME) || (!up && time > 0)) {
            this.counter.play();
        }
        this.updateViewTimes();
        this.updateThumb();
	},

	//Notebook
	fitWidget: function() {
		/* Fit widget around view. */
		var pad = 10;
		var w = this.view.main.width();
		var host = NB.getHostObject();
		this.setAnnRect(host.x, host.y, w + pad, w / this.view.aspectRatio() + pad);
	},

	setAnnRect: function(x, y, width, height) {
		var host = NB.getHostObject();
		NB.undomanager.setAutoRecord(false);
		NB.batch(function() {
			host.x = x;
			host.y = y;
			host.width = width;
			host.height = height;
		});
		NB.undomanager.setAutoRecord(true);
	},

	moveView: function(dx, dy, newWidth, newHeight) {
		/* Move annotation in NB. */
		var dw = newWidth - this.view.main.width();
		var dh = newHeight - this.view.main.height();
		var host = NB.getHostObject();
		this.setAnnRect(host.x + dx, host.y + dy, host.width + dw, host.height + dh);
	},

	updateThumb: function() {
		setTimeout(function() {
			NB.page.updateThumbnail();
		}, 200);
	},

	updateMinSizes: function() {
		var w = 0;
		var h = 1;
		var settings = this.view.areSettingsShown();
		if(this.view.isRectShown()) {
			w = settings ? 190 : 180;
			h = settings ? 80 : 70;
		} else {
			w = settings ? 180 : 170;
		}
		NB.getHostObject().setWidgetMinSize(w, h);
	},

	onSelectionChange: function(selection, isSelected) {
		if(!this.view.areSettingsShown()) {
			this.view.settingsBtn.setShownAnim(isSelected);
		}
	},

	//Persist
	read: function() {
		var strData = NB.persist.load('data');
		if(!strData) {
			return false;
		}
		var data = JSON.parse(strData);
		if(data instanceof Object && data.version === 1) {
			var v = this.view;
			v.setRectShown(data.isRect);
			v.setSettingsShown(data.isSettings);
			this.setCountingUp(data.isCountingUp);
			if(typeof data.lastSetTime === 'number' && data.lastSetTime >= 0) {
				this.lastSetTime = data.lastSetTime;
			}
			if(this.isValidColor(data.color)) {
				v.setColor(data.color);
			}
			v.alertDrop.setSelectedIndex(data.alertIndex);
			v.actionDrop.setSelectedIndex(data.actionIndex);
			this.readTimes(data);
			this.updateViewTimes();
			return true;
		}
		return false;
	},

	write: function() {
		var data = {
			version: 1,
			isRect: this.view.isRectShown(),
			isSettings: this.view.areSettingsShown(),
			isPlaying: this.counter.isPlaying(),
			isCountingUp: this.counter.isCountingUp(),
			time: this.counter.time(),
			lastSetTime: this.lastSetTime,
			clockTime: millis(),
			color: this.view.color(),
			alertIndex: this.view.alertDrop.selectedIndex(),
			actionIndex: this.view.actionDrop.selectedIndex(),
		};
		NB.persist.save('data', data);
	},

	readTimes: function(data) {
		if((typeof data.time === 'number') && data.time >= 0) {
			if(data.isPlaying && (typeof data.clockTime === 'number')) {
				var dt = millis() - data.clockTime;
				if(!data.isCountingUp) {
					dt = - dt;
				}
				var newTime = data.time + dt;
				if(newTime >= Timer.MAX_TIME) {
					this.counter.setTime(Timer.MAX_TIME);
				} else if(newTime <= 0) {
					this.counter.setTime(0);
				} else {
					this.counter.setTime(newTime);
					this.counter.play();
				}
			} else {
				this.counter.setTime(data.time);
			}
		}
	},

	isValidColor: function(rgb) {
		if(!(rgb instanceof Array) || rgb.length !== 3) {
			return false;
		}
		for(var i = 0; i < 3; i++) {
			if(typeof rgb[i] !== 'number') {
				return false;
			}
		}
		return true;
	},

	//Flash
	readFlash: function() {
		var flashString = NB.persist.load('updateData');
		if(flashString) {
			var time = 0; //ms
			var isCountingUp = false;
			var action = 0;
			try {
				var perData = new XmlDocument(flashString);
				var per = perData.childNamed('PersistenceProperties');

				//Time
				time += 600000 * Number(per.childNamed('min1').val);
				time += 60000 * Number(per.childNamed('min2').val);
				time += 10000 * Number(per.childNamed('sec1').val);
				time += 1000 * Number(per.childNamed('sec2').val);
				if(per.childNamed('hr')) {
					//Circle
					time += 3600000 * Number(per.childNamed('hr').val);
				} else {
					//Rectangle
					time += 3600000 * Number(per.childNamed('hr2').val);
					var hr1 = Number(per.childNamed('hr1').val);
					if(hr1 > 0) {
						time = Timer.MAX_TIME;
					}
				}

				//Others
				isCountingUp = per.childNamed('countDown').val === 'false';
				action = Number(per.childNamed('varActionSelectedCombo').val);
			} catch(e) {}
			
			//Bound Action
			if(typeof action !== 'number' || action < 0 || action > 4) {
				action = 0;
			}

			//Bound Time
			if(time < 0) {
				time = 0;
			} else if(time > Timer.MAX_TIME) {
				time = Timer.MAX_TIME;
			}

			//Ratio
			var host = NB.getHostObject();
			var ratio = host.width / host.height;
			return {
				time: time,
				isCountingUp: isCountingUp,
				action: action,
				isRect: ratio > 2,
			}
		}
		return null;
	},

	applyFlash: function(data) {
		//View
		this.view.actionDrop.setSelectedIndex(data.action);
		this.counter.setTime(data.time);
		this.lastSetTime = data.time;
		this.updateViewTimes();
		this.view.setRectShown(data.isRect);
		this.setCountingUp(data.isCountingUp);
		
		//Position
		var host = NB.getHostObject();
		if(data.isRect) {
			this.setAnnRect(host.x, host.y, 276, 100);
		} else {
			this.setAnnRect(host.x, host.y, 200, 200);
		}
	},

	//Rect/CircleView Events
	onCircleButton: function(circle, button, event) {
		if(circle.eventIsInBounds(event)) {
			if(button === circle.play) {
				this.onPlayClick();
			} else if(button === circle.pause) {
				this.onPauseClick();
			} else {
				this.onResetClick();
			}
		}
	},

	onPlayClick: function() {
		this.play();
	},

	onPauseClick: function() {
		this.counter.stop();
		this.updateViewTimes();
		this.updateThumb();
	},

	onResetClick: function() {
		this.counter.stop();
		if(this.counter.isCountingUp()) {
			this.lastSetTime = 0;
			this.counter.setTime(0);
		} else {
			if(this.counter.time() === this.lastSetTime) {
				this.lastSetTime = 0;
				this.counter.setTime(0);
			} else {
				this.counter.setTime(this.lastSetTime);
			}
		}
		this.updateViewTimes();
		this.updateThumb();
	},

	onDropClick: function(drop, event) {
		event.stopPropagation();
		//Hide all others except me
		var shouldShow = !drop.isOptionsShown();
		this.view.hideOptions();
		drop.setOptionsShown(shouldShow);
		this.updateThumb();
	},

	onDropOptionClick: function(drop, index, event) {
		event.stopPropagation();
		drop.setSelectedIndex(index);
		drop.setOptionsShown(false);
		if(drop === this.view.countDrop) {
			this.counter.setCountingUp(index === 1);
			this.view.setProgShown(index === 0);
			this.lastSetTime = this.counter.time();
			this.updateViewTimes();
		} else if(drop === this.view.styleDrop) {
			if(!this.resizeAction) {
				this.resizeAction = new ResizeAction(this, true, index === 0);
			}
			
		}
		this.write();
		this.updateThumb();
	},

	onArrowMouseDown: function(digital, index) {
		this.longPress.mousedown(index);
	},

	onArrowMouseLeave: function(digital, index) {
		this.longPress.mouseup();
	},

	onArrowMouseUp: function(digital, index) {
		if(!this.longPress.isPastFirstDelay()) {
			this.incrementTime(index);
		}
		this.longPress.mouseup();
		this.updateThumb();
	},

	onInputKeyDown: function(digital, event) {
		var isChanged = digital.handleKeyDown(event);
		if(isChanged) {
			var hours = digital.hours();
			var mins = digital.mins();
			var secs = digital.secs();
			this.counter.setTime2(hours, mins, secs, 0);
			this.counter.bound(0, Timer.MAX_TIME);
			this.lastSetTime = this.counter.time();
			this.view.otherDigital(digital).setTime(hours, mins, secs);
			this.updateProg();
			this.write();
			this.updateThumb();
		}
	},

	onInputMouseDown: function(digital, event) {
		this.counter.stop();
		this.updateViewTimes();
		this.updateThumb();
	},

	onInputFocusOut: function(digital, event) {
		digital.fillSpaces();
	},

	//Drag Events
	onHandMouseDown: function() {
		console.log("onHandMouseDown");
		this.handDown = true;
		this.wasPlayingPreDrag = this.counter.isPlaying();
		this.counter.stop();
	},

	onMouseUp: function() {
		if(this.handDown && this.wasPlayingPreDrag) {
			this.play();
		}
		this.wasPlayingPreDrag = false;
		this.handDown = false;		
		this.updateThumb();
	},

	onMouseMove: function(event) {
		if(this.handDown) {
			var newProg;
			if(this.view.isRectShown()) {
				var rect = this.view.rect;
				var offset = rect.progBack.offset();
				var x = event.pageX - offset.left;
				newProg = 1 - (x / rect.progBack.width());
			} else {
				var cir = this.view.circle;
				var offset = cir.main.offset();
				var x = event.pageX - offset.left;
				var y = event.pageY - offset.top;
				var newAngle = cir.degreesFor(x, y);
				newProg = cir.progFor(newAngle);
			}
			if(newProg > 1) {
				newProg = 1;
			} else if(newProg < 0) {
				newProg = 0;
			}
			this.counter.setTime(newProg * this.lastSetTime);
			var ceil = this.counter.ceilTime();
			this.view.setTime(ceil.hours, ceil.mins, ceil.secs);
			this.updateProg();
			this.write();
		}
	},
	

	//Color Picker Events
	onColorBox: function(event) {
		this.view.toggleColorPickerShown();
		event.stopPropagation();
		this.updateThumb();
	},

	onColorPickerX: function(picker) {
		this.view.setColorPickerShown(false);
		this.updateThumb();
	},

	onColorPickerHex: function(picker, newRgb) {
		this.setColor(newRgb);
	},

	onColorPickerWheelUp: function(picker, newRgb) {
		this.setColor(newRgb);
	},

	onColorPickerDrag: function(picker, newRgb) {
		this.setColor(newRgb);
	},

	//Other Events
	onResize: function() {
		this.view.layout();
		setTimeout(this.view.layout.bind(this.view), 10); //In the case where improper window coords are sent to Widget
	},
	
	onSettingsClick: function() {
		if(!this.resizeAction) {
			this.resizeAction = new ResizeAction(this, false);
		}
	},

	onMainClick: function() {
		this.view.hideOptions();
		this.updateThumb();
	},

	onCounterUpdate: function() {
		var c = this.counter;
		if(c.time() < 0) {
			//ran out of time.
			c.stop();
			c.setTime(0);
			this.timesUp();
		} else if(c.time() > Timer.MAX_TIME) {
			c.stop();
			c.setTime(Timer.MAX_TIME);
		}
		this.updateViewTimes();
	},

	onLongPressInc: function(longPress, btnIndex) {
		this.incrementTime(btnIndex);
	},

	onFocusIn: function() {
		this.view.settingsBtn.setShownAnim(true);
	},

	onFocusOut: function() {
		if(!this.view.areSettingsShown() && !this.selection.isSelected()) {
			this.view.settingsBtn.setShownAnim(false);
		}
	},

	//Other
	incrementTime: function(arrowBtnIndex) {
		var c = this.counter;
		switch(arrowBtnIndex) {
		case 0:
			c.addHours(1);
			break;
		case 1:
			c.addMins(1);
			break;
		case 2:
			c.addSecs(1);
			break;
		case 3:
			c.addHours(-1);
			break;
		case 4:
			c.addMins(-1);
			break;
		case 5:
			c.addSecs(-1);
			break;
		}
		c.bound(0, Timer.MAX_TIME);
		if(!this.counter.isPlaying()) {
			this.lastSetTime = this.counter.time();
		}
		this.updateViewTimes();
	},

	timesUp: function() {
		/* Triggered when clock runs out. */
		this.playSound();
		setTimeout(this.triggerAction.bind(this), 2000);
	},

	playSound: function() {
		var files = [
			'sound/Bell1.mp3', //Order matters
			'sound/Bell2.mp3',
			'sound/Bell3.mp3',
			'sound/Applause.mp3',
		];
		var i = this.view.alertDrop.selectedIndex() - 1;
		var duration = 5000;
		if(i !== -1) {
			this.aud.setAttribute('src', files[i]);
			this.aud.play();
			setTimeout(this.stopSound.bind(this), duration);
		} else {
			this.aud.setAttribute('src', '');
		}
	},

	stopSound: function() {
		this.aud.setAttribute('src', '');
	},

	triggerAction: function() {
		try { //in case pages dont exist.
			var i = this.view.actionDrop.selectedIndex();
			var ids = NB.document.getPageIds();
			if(i === 1) {
				NB.document.viewPage(ids[0]);
			} else if(i === 2) {
				NB.document.viewPage(ids[ids.length - 1]);
			} else if(i === 3) {
				NB.document.viewNextPage();
			} else if(i === 4) {
				NB.document.viewPreviousPage();
			}
		} catch(e) {}
	},

	toggleSettings: function() {
		//Resize Widget
		var settingsShown = this.view.areSettingsShown();
		var ratio = this.view.aspectRatioFor(this.view.isRectShown(), !settingsShown);
		var width = this.view.main.width();
		if(this.view.isRectShown()) {
			var x1 = 1 / (1 - 2 * this.view.settingsPad);
			var wRatio = x1 / 1.099; //(width of settings shown) / (width view of settings hidden)
			var newWidth;
			var dx;
			if(settingsShown) {
				newWidth = width / wRatio;
				dx = this.view.settingsPad * width;
			} else {
				newWidth = width * wRatio;
				dx = -this.view.settingsPad * newWidth;
			}
			this.moveView(dx, dx, newWidth, newWidth / ratio);
		} else {
			if(settingsShown) {
				var dx = this.view.settingsPad * width;
				var newWidth = width * (1 - 2 * this.view.settingsPad);
				this.moveView(dx, dx, newWidth, newWidth);
			} else {
				var newWidth = width / (1 - 2 * this.view.settingsPad);
				var dx = -this.view.settingsPad * newWidth;
				this.moveView(dx, dx, newWidth, newWidth / ratio);
			}
		}

		//Relayout
		this.view.setSettingsShown(!settingsShown);
		this.write();
		this.updateMinSizes();
	},

	setRectShown: function(shown) {
		this.view.setRectShown(shown);
		this.fitWidget();
		this.write();
		this.updateMinSizes();
	},

};

//Constants
Timer.MAX_TIME = 35999000; //ms corresponding to 9:59:59

//Init
NB.ready(function() {
	if (window.requiresNewerNBSDKVersionError) {
        var container = window.document.createElement("div");
        container.setAttribute("id", "requiresNewerNBSDKVersionErrorContainer");

        var p1 = window.document.createElement("p");
        p1.textContent = NB.localize("This activity requires a later version of SMART Notebook software.");
        container.appendChild(p1);

        var p2 = window.document.createElement("p");
        p2.textContent = NB.localize("Make sure you're using Notebook software version %d or later.").replace("%d", "16");
        container.appendChild(p2);

        document.body.appendChild(container);
        document.body.style.backgroundColor = "lightgray";

        function adjustHeights(id) {
            var elem = document.getElementById(id);
            var fontstep = 2;
            if (elem.offsetHeight > document.documentElement.clientHeight) {
                elem.style.fontSize = ((window.getComputedStyle(elem).fontSize.substr(0,2)-fontstep).toString() + 'px');
                elem.style.lineHeight = (window.getComputedStyle(elem).fontSize.substr(0,2).toString() + 'px');
                adjustHeights(id);
            }
        }

        adjustHeights("requiresNewerNBSDKVersionErrorContainer");

        return;
    }
	x = new Timer();
});
