/* 
Main view object. Wraps all the other view objects.
*/

var TimerView = function() {
	//Properties
	this.settingsBtn = new SettingsButton('img/gear.png');
	this.rect = new RectView();
	this.circle = new CircleView();
	this.circle.main.hide();
	this.colorPicker = new ColorPicker();
	this.colorPicker.jq.addClass('color-picker');
	this.colorPicker.setHexBorder(0.06, '#585858');
	this.colorTriangle = $('<div/>').addClass('color-triangle');
	this.styleDrop = new DropView(strs.style, [strs.rect, strs.circle]);
	this.actionDrop = new DropView(strs.action, 
		[strs.nothing, strs.firstPage, strs.lastPage, strs.nextPage, strs.prevPage]);
	this.alertDrop = new DropView(strs.alertSounds,
		[strs.none, strs.bell1, strs.bell2, strs.bell3, strs.applause]);
	this.countDrop = new DropView(strs.counter, [strs.countDown, strs.countUp]);
	this.drops = [this.styleDrop, this.actionDrop, this.alertDrop, this.countDrop];
	this.colorBox = new DropView(strs.color, []);
	this.colorBox.triangle.hide();
	this.back = $('<div/>')
		.addClass('timer-view-back')
	this.settings = $('<div/>')
		.addClass('settings')
		.append(this.colorBox.main, this.styleDrop.main, this.actionDrop.main, 
			this.alertDrop.main, this.countDrop.main, this.colorTriangle, this.colorPicker.jq);
	this.main = $('<div/>')
		.addClass('timer-view')
		.append(this.back, this.settingsBtn.jq, this.rect.main, this.circle.main, this.settings);
	this._settingsShown = false;
	this._color = [71, 71, 71]; //rgb
	this.settingsPad = 0.07; //percentage of padding between circleview and edge of view when settings shown.
	this.setSettingsShown(false);
	this.setRectShown(true);
	this.setColorPickerShown(false);
	this.colorPicker.setRgb(this._color);
	this.setColor(this._color);
	this.makeNBClickable();
};

TimerView.prototype = {

	//Props
	setTime: function(hours, mins, secs) {
		this.rect.digital.setTime(hours, mins, secs);
		this.circle.digital.setTime(hours, mins, secs);
	},

	otherDigital: function(digital) {
		if(digital === this.rect.digital) {
			return this.circle.digital;
		}
		return this.rect.digital;
	},

	//Play/Prog
	setIsPlaying: function(val) {
		this.rect.setPlayShown(!val);
		this.circle.setPlayShown(!val);
	},

	setProgShown: function(shown) {
		this.rect.setProgShown(shown);
		this.circle.setProgShown(shown);
		this.rect.layout();
		this.circle.layout();
	},

	setProg: function(prog) {
		if(prog > 1) {
			prog = 1;
		} else if(prog < 0) {
			prog = 0;
		}
		this.circle.setProg(prog);
		this.rect.setProg(prog);
	},

	//Switch Views
	isRectShown: function() {
		return this.styleDrop.selectedIndex() === 0;
	},

	setRectShown: function(shown) {
		this.styleDrop.setSelectedIndex(shown ? 0 : 1);
		if(shown) {
			this.rect.main.show();
			this.circle.main.hide();
		} else {
			this.rect.main.hide();
			this.circle.main.show();
		}
		this.settingsBtn.setShown(true);
		this.layout();
	},

	//Color
	color: function() {
		return this._color;
	},

	setColor: function(rgb) {
		/* 
		Input
			rgb: [r, g, b] 0-255
		*/
		this._color = rgb;
		var str = ColorPicker.rgbToString(rgb);
		this.rect.progBack.css('backgroundColor', str);
		this.rect.inner.css('backgroundColor', str);
		this.circle.v1.css('backgroundColor', str);
		this.colorBox.box.css('backgroundColor', str);

		//Set alternate color to dark/light depending on how bright background is.
		var showLight = !ColorPicker.isDarkText(rgb);
		var alternate = showLight ? 'white' : 'rgb(71,71,71)';
		this.rect.setAlternateColor(alternate);
		this.rect.setLightBtnsShown(showLight);
		this.circle.setInnerColor(alternate);
		this.circle.setLightBtnsShown(showLight);
	},

	isColorPickerShown: function() {
		return this.colorPicker.jq.is(':visible');
	},

	setColorPickerShown: function(shown) {
		if(shown) {
			this.colorPicker.jq.show();
			this.colorTriangle.show();
		} else {
			this.colorPicker.jq.hide();
			this.colorTriangle.hide();
		}
	},

	toggleColorPickerShown: function() {
		var shown = this.colorPicker.jq.is(':visible');
		this.setColorPickerShown(!shown);
	},

	//Settings
	areSettingsShown: function() {
		return this._settingsShown;
	},
	
	setSettingsShown: function(shown) {
		this._settingsShown = shown;
		this.settingsBtn.setShown(true);
		if(shown) {
			this.settings.show();
			this.back.show();
		} else {
			this.settings.hide();
			this.back.hide();
		}
		this.settingsBtn.back.css({
			backgroundColor: shown ? 'transparent' : '#03bede',
		});
		this.layout();
	},

	toggleSettings: function() {
		this.setSettingsShown(!this._settingsShown);
	},
	
	//Layout
	layout: function() {
		var width = this.main.width();
		this.layoutSettingsBtn();
		this.updateBorders();
		var marg = this.settingsPad * width;
		if(this.isRectShown()) {
			this.layoutRect();
			if(this._settingsShown) {
				this.layoutSettings(this.rect.main.position().top + marg + this.rect.main.height());
			}
		} else {
			this.layoutCircle();
			if(this._settingsShown) {
				this.layoutSettings(2 * marg + this.circle.main.height());
			}
		}
	},

	layoutRect: function() {
		var wMain = this.main.width();
		var hMain = this.main.height();
		var marg = this.settingsPad * wMain;
		var border = 2;
		var left = 0;
		var top = 0;
		var width = 0;
		var height = 0;
		if(this._settingsShown) {
			left = marg;
			top = 0.12 * wMain;
			width = wMain - 2 * marg;
			height = wMain / 3;
		} else {
			left = 0.02 * wMain;
			top = 0.02 * hMain;
			width = wMain - left - 0.8 * this.settingsBtn.jq.width();
			height = 0.96 * hMain - border;
		}
		this.rect.main.css({
			left: left,
			top: top,
			width: width,
			height: height,
		});
		this.rect.layout();
	},

	layoutCircle: function() {
		var wMain = this.main.width();
		var marg = this._settingsShown ? this.settingsPad * wMain : 1;
		var w = wMain - 2 * marg;
		this.circle.main.css({
			left: marg,
			top: marg,
			width: w,
			height: w,
		});
		this.circle.layout();
	},

	layoutSettingsBtn: function() {
		var btn = this.settingsBtn;
		if(this._settingsShown) {
			btn.layoutInCorner(this.main.width());
		} else {
			if(this.isRectShown()) {
				btn.layoutBehindRect(this.main.height(), 0.02 * this.main.height() + 1);
			} else {
				btn.layoutBehindCircle(this.main.width());
			}
		}
	},

	setDimens: function(x, y) {
		this.main.width(x).height(y);
	},

	layoutSettings: function(top) {
		this.settings.css({
			top: top,
		});
		var w = this.main.width();
		var width = 0.38 * w;
		var height = 0.075 * w;
		var x1 = this.settingsPad * w;
		var x2 = w - width - x1;
		var y1 = 0.05 * w;
		var h = 0.18 * w;
		
		this.alertDrop.layoutLTWH(x1, y1, width, height);
		this.countDrop.layoutLTWH(x2, y1, width, height);
		this.styleDrop.layoutLTWH(x1, y1 + h, width, height);
		this.actionDrop.layoutLTWH(x2, y1 + h, width, height);
		this.colorBox.layoutLTWH(x1, y1 + 2 * h, width, height);
		this.layoutColorViews();
	},

	layoutColorViews: function() {
		var pos = this.colorBox.main.position();
		var x1 = pos.left + this.colorBox.main.width() * 1.04;
		var y1 = pos.top + this.colorBox.main.height() / 2;
		var w1 = this.colorTriangle.width() / 2;
		this.colorTriangle.css({
			left: x1 - w1,
			top: y1 - w1,
		});
		var w2 = this.colorPicker.jq.width();
		this.colorPicker.jq.css({
			left: x1,
			top: pos.top - 0.8 *  w2,
			height: w2,
		});
		this.colorPicker.layout({
			wheelWidth: 0.9,
			wheelThickness: 0.15,
			hexWidth: 0.15,
			hexRadius: 0.18,
		});
	},

	updateBorders: function() {
		var b = 0.01 * this.main.width();
		if(b > 2) {
			b = 2;
		} else if(b < 1) {
			b = 1;
		}
		var elements = [
			this.styleDrop, 
			this.actionDrop, 
			this.alertDrop, 
			this.countDrop, 
			this.colorBox
		];
		for(var i = 0; i < elements.length; i++) {
			elements[i].borderWidth = b;
		}
	},

	aspectRatio: function() { // w / h
		return this.aspectRatioFor(this.isRectShown(), this._settingsShown);
	},

	aspectRatioFor: function(rectShown, settingsShown) {
		if(rectShown) {
			if(settingsShown) {
				return 0.84;
			} else {
				return 3;
			}
		} else {
			if(settingsShown) {
				return 0.6;
			} else {
				return 1;
			}
		}
	},

	//Other
	makeNBClickable: function() {
		var cir = this.circle;
		var rect = this.rect;
		var jqElements = [
			cir.digital.main,
			cir.v1,
			cir.v2,
			cir.wedge.jq,
			cir.main,
			rect.digital.main,
			rect.inner,
			rect.main,
			this.settings,
			this.back,
			this.main,
		];
		for(var i = 0; i < jqElements.length; i++) {
			jqElements[i].addClass('smart-nb-draggable smart-nb-clickable smart-nb-selectable');
		}
	},

	hideOptions: function() {
		this.styleDrop.setOptionsShown(false);
		this.actionDrop.setOptionsShown(false);
		this.alertDrop.setOptionsShown(false);
		this.countDrop.setOptionsShown(false);
		this.setColorPickerShown(false);
	},

	setArrowsShown: function(arrowsShown) {
		this.rect.digital.setArrowsShown(arrowsShown);
		this.circle.digital.setArrowsShown(arrowsShown);
	},

};
